@@ -52,6 +52,10 @@ class Mesh(BaseMesh):
     unify_cycles = mesh_unify_cycles
     quads_to_triangles = mesh_quads_to_triangles
 
+    def transform_numpy(self, M):
+        from compas.datastructures.mesh.transformations_numpy import mesh_transform_numpy
+        mesh_transform_numpy(self, M)
+
     def to_trimesh(self):
         # convert to mesh with only triangle faces
         # provides options that define the rules for triangulation
