@@ -1,8 +1,9 @@
 from __future__ import print_function
+from __future__ import absolute_import
+from __future__ import division
 
 from functools import wraps
 
-import compas
 import compas_rhino
 
 from compas.geometry import centroid_polygon
@@ -11,52 +12,50 @@ from compas_rhino.utilities import create_layers_from_path
 from compas_rhino.utilities import clear_layer
 from compas_rhino.utilities import clear_current_layer
 
-if compas.IPY:
-    from System.Collections.Generic import List
-    from System.Drawing.Color import FromArgb
-    from System.Enum import ToObject
+from System.Collections.Generic import List
+from System.Drawing.Color import FromArgb
+from System.Enum import ToObject
 
-    if compas.RHINO:
-        import rhinoscriptsyntax as rs
-        import scriptcontext as sc
+import rhinoscriptsyntax as rs
+import scriptcontext as sc
 
-        from Rhino.Geometry import Point3d
-        from Rhino.Geometry import Vector3d
-        from Rhino.Geometry import Polyline
-        from Rhino.Geometry import PolylineCurve
-        from Rhino.Geometry import GeometryBase
-        from Rhino.Geometry import Brep
-        from Rhino.Geometry import Cylinder
-        from Rhino.Geometry import Circle
-        from Rhino.Geometry import Plane
-        from Rhino.Geometry import PipeCapMode
-        from Rhino.Geometry import Curve
-        from Rhino.Geometry import Sphere
-        from Rhino.Geometry import TextDot
-        from Rhino.Geometry import Mesh as RhinoMesh
-        from Rhino.DocObjects.ObjectColorSource import ColorFromObject
-        from Rhino.DocObjects.ObjectColorSource import ColorFromLayer
-        from Rhino.DocObjects.ObjectDecoration import EndArrowhead
-        from Rhino.DocObjects.ObjectDecoration import StartArrowhead
-        from Rhino.DocObjects.ObjectPlotWeightSource import PlotWeightFromObject
+from Rhino.Geometry import Point3d
+from Rhino.Geometry import Vector3d
+from Rhino.Geometry import Polyline
+from Rhino.Geometry import PolylineCurve
+from Rhino.Geometry import GeometryBase
+from Rhino.Geometry import Brep
+from Rhino.Geometry import Cylinder
+from Rhino.Geometry import Circle
+from Rhino.Geometry import Plane
+from Rhino.Geometry import PipeCapMode
+from Rhino.Geometry import Curve
+from Rhino.Geometry import Sphere
+from Rhino.Geometry import TextDot
+from Rhino.Geometry import Mesh as RhinoMesh
+from Rhino.DocObjects.ObjectColorSource import ColorFromObject
+from Rhino.DocObjects.ObjectColorSource import ColorFromLayer
+from Rhino.DocObjects.ObjectDecoration import EndArrowhead
+from Rhino.DocObjects.ObjectDecoration import StartArrowhead
+from Rhino.DocObjects.ObjectPlotWeightSource import PlotWeightFromObject
 
-        find_object = sc.doc.Objects.Find
-        add_point = sc.doc.Objects.AddPoint
-        add_line = sc.doc.Objects.AddLine
-        add_dot = sc.doc.Objects.AddTextDot
-        add_curve = sc.doc.Objects.AddCurve
-        add_polyline = sc.doc.Objects.AddPolyline
-        add_brep = sc.doc.Objects.AddBrep
-        add_sphere = sc.doc.Objects.AddSphere
-        add_mesh = sc.doc.Objects.AddMesh
-        add_circle = sc.doc.Objects.AddCircle
+find_object = sc.doc.Objects.Find
+add_point = sc.doc.Objects.AddPoint
+add_line = sc.doc.Objects.AddLine
+add_dot = sc.doc.Objects.AddTextDot
+add_curve = sc.doc.Objects.AddCurve
+add_polyline = sc.doc.Objects.AddPolyline
+add_brep = sc.doc.Objects.AddBrep
+add_sphere = sc.doc.Objects.AddSphere
+add_mesh = sc.doc.Objects.AddMesh
+add_circle = sc.doc.Objects.AddCircle
 
-        TOL = sc.doc.ModelAbsoluteTolerance
+TOL = sc.doc.ModelAbsoluteTolerance
 
-        try:
-            find_layer_by_fullpath = sc.doc.Layers.FindByFullPath
-        except SystemError:
-            find_layer_by_fullpath = None
+try:
+    find_layer_by_fullpath = sc.doc.Layers.FindByFullPath
+except SystemError:
+    find_layer_by_fullpath = None
 
 
 __all__ = [
@@ -731,18 +730,26 @@ def draw_mesh(vertices, faces, name=None, color=None, disjoint=False, **kwargs):
     str or GUID
 
     """
-    points = []
     mesh = RhinoMesh()
     if disjoint:
-        for keys in faces:
-            i = len(points)
-            facet = [j + i for j in range(len(keys))]
-            for key in keys:
-                point = vertices[key]
-                points.append(point)
-                x, y, z = point
-                mesh.Vertices.Add(x, y, z)
-            mesh.Faces.AddFace(*facet)
+        P = 0
+        for face in faces:
+            f = len(face)
+            if f == 3:
+                mesh.Vertices.Add(* vertices[face[0]])
+                mesh.Vertices.Add(* vertices[face[1]])
+                mesh.Vertices.Add(* vertices[face[2]])
+                mesh.Faces.AddFace(P + 0, P + 1, P + 2)
+                P += 3
+            elif f == 4:
+                mesh.Vertices.Add(* vertices[face[0]])
+                mesh.Vertices.Add(* vertices[face[1]])
+                mesh.Vertices.Add(* vertices[face[2]])
+                mesh.Vertices.Add(* vertices[face[3]])
+                mesh.Faces.AddFace(P + 0, P + 1, P + 2, P + 3)
+                P += 4
+            else:
+                continue
     else:
         for x, y, z in vertices:
             mesh.Vertices.Add(x, y, z)
